@@ -4,15 +4,10 @@ import datetime
 from uuid import UUID, uuid1
 from typing import Dict, Any, Optional, Tuple, Callable
 from importlib.metadata import version
-import urllib.request
-import urllib.response
-import urllib.parse
-from urllib.error import URLError, HTTPError
-from socket import timeout
-import ssl
 
 from rkclient.auth import RK_USER_AUTH_HEADER, RK_PUC_AUTH_HEADER
 from rkclient.entities import PEM, Artifact
+from rkclient.request import RequestHelper
 from rkclient.serialization import PEMSerialization, _encode_as_base64
 
 log = logging.getLogger("rkclient")
@@ -62,24 +57,19 @@ class RKClient:
         :param timeout_sec:
         :param insecure: set it to True when operating with server that has test SSL certificates
         """
-        self.receiver_url = receiver_url.rstrip('/')
+
+        receiver_url = receiver_url.rstrip('/')
+        self.receiver_request = RequestHelper(receiver_url, timeout_sec, insecure, user_auth, puc_auth)
+        log.info(f"ver {RK_VERSION}, connecting to: {receiver_url}")
+
         if emitter_id is None:
             emitter_id = uuid1()
         self.emitter_id = emitter_id
-        self.timeout_sec = timeout_sec
-        self.user_auth = user_auth
-        self.puc_auth = puc_auth
-        log.info(f"ver {RK_VERSION}, connecting to: {self.receiver_url}")
-        if self.user_auth:
+
+        if user_auth:
             log.info(f"Authorizing with {RK_USER_AUTH_HEADER} header")
-        elif self.puc_auth:
+        elif puc_auth:
             log.info(f"Authorizing with {RK_PUC_AUTH_HEADER} header")
-        self.insecure = insecure
-        if self.insecure:
-            log.warning("Disabled SSL certificate check")
-        if "https://" not in self.receiver_url and "http://receiver:8083" not in self.receiver_url and \
-                "localhost" not in self.receiver_url and "127.0.0.1" not in self.receiver_url:
-            log.warning("If you're connecting to https server without specifying https in url, expect 405 errors")
 
     def __enter__(self):
         return self
@@ -145,7 +135,7 @@ class RKClient:
         def _send_pem():
             payload: str = PEMSerialization.to_json(pem)
             log.debug(f"sending PEM: {payload}")
-            return self._post("/pem", payload)
+            return self.receiver_request.post("/pem", payload)
 
         return _handle_request(_send_pem, "Sending PEM")
 
@@ -154,7 +144,7 @@ class RKClient:
         :return: check class description
         """
         def _ping():
-            return self._get("/ping")
+            return self.receiver_request.get("/ping")
 
         return _handle_request(_ping, "Pinging")
 
@@ -165,7 +155,7 @@ class RKClient:
         TODO: return object instead of str
         """
         def _get_info():
-            return self._get("/info", )
+            return self.receiver_request.get("/info", )
 
         return _handle_request(_get_info, "Getting info", True)
 
@@ -176,7 +166,7 @@ class RKClient:
         """
         def _get_tag():
             tag_base64 = _encode_as_base64(tag_name)
-            return self._get(f"/tag/{namespace.hex}/{tag_base64}")
+            return self.receiver_request.get(f"/tag/{namespace.hex}/{tag_base64}")
 
         return _handle_request(_get_tag, "Getting tag", True)
 
@@ -190,54 +180,10 @@ class RKClient:
         """
         def _set_tag():
             tag_base64 = _encode_as_base64(tag_name)
-            return self._post(f"/tag/{namespace.hex}/{tag_base64}/{pem.ID.hex}", tag_base64)
+            return self.receiver_request.post(f"/tag/{namespace.hex}/{tag_base64}/{pem.ID.hex}", tag_base64)
 
         return _handle_request(_set_tag, "Setting tag")
 
-    def _get(self, url_postfix: str, query_params: Optional[Dict[str, Any]] = None) -> Tuple[str, bool]:
-        url = self.receiver_url + url_postfix + self._encode_query_params(query_params)
-        req = urllib.request.Request(url=url)
-        if self.user_auth:
-            req.add_header(RK_USER_AUTH_HEADER, self.user_auth)
-        elif self.puc_auth:
-            req.add_header(RK_PUC_AUTH_HEADER, self.puc_auth)
-        return self._make_request(req)
-
-    def _post(self, url_postfix: str, payload: str, query_params: Optional[Dict[str, Any]] = None) -> Tuple[str, bool]:
-        url = self.receiver_url + url_postfix + self._encode_query_params(query_params)
-        req = urllib.request.Request(url=url, data=payload.encode())
-        req.add_header('Content-Type', 'application/json')
-        if self.user_auth:
-            req.add_header(RK_USER_AUTH_HEADER, self.user_auth)
-        elif self.puc_auth:
-            req.add_header(RK_PUC_AUTH_HEADER, self.puc_auth)
-        return self._make_request(req)
-
-    def _make_request(self, request):
-        try:
-            resp = urllib.request.urlopen(request, timeout=self.timeout_sec, context=self._get_ssl_context())
-        except HTTPError as e:
-            return f"error: {e} {e.read().decode()}", False
-        except URLError as e:
-            return f"connection error: {e}", False
-        except timeout as e:
-            return f"socket timed out in {self.timeout_sec}s: {e}", False
-        else:
-            return resp.read().decode(), True
-
-    def _get_ssl_context(self) -> ssl.SSLContext:
-        ctx = ssl.create_default_context()
-        if self.insecure:
-            ctx.check_hostname = False
-            ctx.verify_mode = ssl.CERT_NONE
-        return ctx
-
-    def _encode_query_params(self, query_params: Optional[Dict[str, Any]]) -> str:
-        if query_params is None or len(query_params) == 0:
-            return ''
-        encoded_params = urllib.parse.urlencode(query_params)
-        return '?' + encoded_params
-
 
 def _handle_request(func: Callable, name: str, ret_text_on_ok: bool = False) -> Tuple[str, bool]:
     """
@@ -252,3 +198,4 @@ def _handle_request(func: Callable, name: str, ret_text_on_ok: bool = False) ->
         return text, True
     else:
         return 'OK', True
+
